package com.zoinla.flir_plugin

import android.content.Context
import android.content.SharedPreferences
import android.net.Uri
import androidx.annotation.VisibleForTesting
import io.flutter.plugin.common.MethodCall

class FlirCache(context: Context) {
    private var prefs: SharedPreferences = context.getSharedPreferences(SHARED_PREFERENCES_NAME, Context.MODE_PRIVATE)

    fun saveDimensionWithMethodCall(methodCall: MethodCall) {
        val maxWidth = methodCall.argument<Double>(MAP_KEY_MAX_WIDTH)
        val maxHeight = methodCall.argument<Double>(MAP_KEY_MAX_HEIGHT)
        val imageQuality = methodCall.argument<Int>(MAP_KEY_IMAGE_QUALITY) ?: 100

        setMaxDimension(maxWidth, maxHeight, imageQuality)
    }

    private fun setMaxDimension(maxWidth: Double?, maxHeight: Double?, imageQuality: Int) {
        val editor = prefs.edit()
        maxWidth?.let {
            editor.putLong(SHARED_PREFERENCE_MAX_WIDTH_KEY, java.lang.Double.doubleToRawLongBits(maxWidth))
        }
        maxHeight?.let {
            editor.putLong(SHARED_PREFERENCE_MAX_HEIGHT_KEY, java.lang.Double.doubleToRawLongBits(maxHeight))
        }

        if (imageQuality > -1 && imageQuality < 101) {
            editor.putInt(SHARED_PREFERENCE_IMAGE_QUALITY_KEY, imageQuality)
        } else {
            editor.putInt(SHARED_PREFERENCE_IMAGE_QUALITY_KEY, 100)
        }
        editor.apply()
    }

    fun savePendingCameraMediaUriPath(uri: Uri) {
        prefs.edit().putString(SHARED_PREFERENCE_PENDING_IMAGE_URI_PATH_KEY, uri.path).apply()
    }

    fun retrievePendingCameraMediaUriPath() = prefs.getString(SHARED_PREFERENCE_PENDING_IMAGE_URI_PATH_KEY, "")

    fun saveResult(path: String?, errorCode: String?, errorMessage: String?) {
        val editor = prefs.edit()
        path?.let {
            editor.putString(FLIR_IMAGE_PATH_KEY, path)
        }
        errorCode?.let {
            editor.putString(SHARED_PREFERENCE_ERROR_CODE_KEY, errorCode)
        }
        errorMessage?.let {
            editor.putString(SHARED_PREFERENCE_ERROR_MESSAGE_KEY, errorMessage)
        }
        editor.apply()
    }

    fun clear() {
        prefs.edit().clear().apply()
    }

    fun getCacheMap(): Map<String, Any?> {
        val resultMap = HashMap<String, Any?>()
        var hasData = false

        if (prefs.contains(FLIR_IMAGE_PATH_KEY)) {
            val imagePathValue = prefs.getString(FLIR_IMAGE_PATH_KEY, "")
            resultMap[MAP_KEY_PATH] = imagePathValue
            hasData = true
        }

        if (prefs.contains(SHARED_PREFERENCE_ERROR_CODE_KEY)) {
            val errorCodeValue = prefs.getString(SHARED_PREFERENCE_ERROR_CODE_KEY, "")
            resultMap[MAP_KEY_ERROR_CODE] = errorCodeValue
            hasData = true
            if (prefs.contains(SHARED_PREFERENCE_ERROR_MESSAGE_KEY)) {
                val errorMessageValue = prefs.getString(SHARED_PREFERENCE_ERROR_MESSAGE_KEY, "")
                resultMap[MAP_KEY_ERROR_MESSAGE] = errorMessageValue
            }
        }

        if (hasData) {
            if (prefs.contains(SHARED_PREFERENCE_MAX_WIDTH_KEY)) {
                val maxWidthValue = prefs.getLong(SHARED_PREFERENCE_MAX_WIDTH_KEY, 0)
                resultMap[MAP_KEY_MAX_WIDTH] = java.lang.Double.longBitsToDouble(maxWidthValue)
            }
            if (prefs.contains(SHARED_PREFERENCE_MAX_HEIGHT_KEY)) {
                val maxHeightValue = prefs.getLong(SHARED_PREFERENCE_MAX_HEIGHT_KEY, 0)
                resultMap[MAP_KEY_MAX_HEIGHT] = java.lang.Double.longBitsToDouble(maxHeightValue)
            }
            if (prefs.contains(SHARED_PREFERENCE_IMAGE_QUALITY_KEY)) {
                val imageQuality = prefs.getInt(SHARED_PREFERENCE_IMAGE_QUALITY_KEY, 100)
                resultMap[MAP_KEY_IMAGE_QUALITY] = imageQuality
            } else {
                resultMap[MAP_KEY_IMAGE_QUALITY] = 100
            }
        }

        return resultMap
    }

    companion object {
        const val MAP_KEY_PATH = "path"
        const val MAP_KEY_MAX_WIDTH = "maxWidth"
        const val MAP_KEY_MAX_HEIGHT = "maxHeight"
        const val MAP_KEY_IMAGE_QUALITY = "imageQuality"

        private const val MAP_KEY_ERROR_CODE = "errorCode";
        private const val MAP_KEY_ERROR_MESSAGE = "errorMessage";

        private const val FLIR_IMAGE_PATH_KEY = "flir_image_path"

        private const val SHARED_PREFERENCE_ERROR_CODE_KEY = "flir_error_code"
        private const val SHARED_PREFERENCE_ERROR_MESSAGE_KEY = "flir_error_message"
        private const val SHARED_PREFERENCE_MAX_WIDTH_KEY = "flir_max_width"
        private const val SHARED_PREFERENCE_MAX_HEIGHT_KEY = "flir_max_height"
        private const val SHARED_PREFERENCE_IMAGE_QUALITY_KEY = "flir_image_quality"
        private const val SHARED_PREFERENCE_PENDING_IMAGE_URI_PATH_KEY = "flir_pending_image_uri"

        @VisibleForTesting const val SHARED_PREFERENCES_NAME = "flir_shared_preference"
    }
}