package com.zoinla.flir_plugin

import android.content.CursorLoader
import android.content.Intent
import android.database.Cursor
import android.graphics.Bitmap
import android.net.Uri
import android.os.Bundle
import android.os.Environment
import android.provider.MediaStore
import android.provider.OpenableColumns
import android.util.Log
import androidx.appcompat.app.AppCompatActivity
import com.flir.thermalsdk.ErrorCode
import com.flir.thermalsdk.androidsdk.ThermalSdkAndroid
import com.flir.thermalsdk.androidsdk.live.connectivity.UsbPermissionHandler
import com.flir.thermalsdk.live.CommunicationInterface
import com.flir.thermalsdk.live.Identity
import com.flir.thermalsdk.live.connectivity.ConnectionStatusListener
import com.flir.thermalsdk.live.discovery.DiscoveryEventListener
import com.flir.thermalsdk.log.ThermalLog
import com.zoinla.flir_plugin.databinding.ActivityFlirCameraBinding
import java.io.File
import java.io.FileOutputStream
import java.io.IOException
import java.net.URI
import java.util.concurrent.LinkedBlockingQueue


class FlirCameraActivity : AppCompatActivity() {
    private val TAG = "FlirCameraActivity"

    private lateinit var binding: ActivityFlirCameraBinding
    private lateinit var permissionHandler: PermissionHandler
    private lateinit var cameraHandler: CameraHandler

    private var connectedIdentity: Identity? = null

    private val framesBuffer = LinkedBlockingQueue<FrameDataHolder>(21)
    private val usbPermissionHandler = UsbPermissionHandler()

    private var mBitmap: Bitmap? = null

    /**
     * Show message on the screen
     */
    interface ShowMessage {
        fun show(message: String?)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityFlirCameraBinding.inflate(layoutInflater)
        setContentView(binding.root)

        ThermalSdkAndroid.init(
                applicationContext,
                if (BuildConfig.DEBUG) ThermalLog.LogLevel.DEBUG
                else ThermalLog.LogLevel.NONE
        )

        permissionHandler = PermissionHandler(showMessage, this)

        cameraHandler = CameraHandler()
        cameraHandler.startDiscovery(cameraDiscoveryListener, discoveryStatusListener)

        val mediaUri = intent.extras?.get(MediaStore.EXTRA_OUTPUT) as Uri?

        binding.imvCapture.isEnabled = false
        binding.imvCapture.setOnClickListener {
            mediaUri?.let {
                try {
                    contentResolver.query(it, null, null, null, null)?.use { cursor ->
                        val nameIndex = cursor.getColumnIndex(OpenableColumns.DISPLAY_NAME)
                        cursor.moveToFirst()
                        val file = File(getExternalFilesDir(Environment.DIRECTORY_PICTURES).toString() + "/" + cursor.getString(nameIndex))
                        val fOut = FileOutputStream(file)
                        mBitmap?.compress(Bitmap.CompressFormat.PNG, 85, fOut)
                        fOut.flush()
                        fOut.close()

                        val data = Intent()
                        data.data = mediaUri
                        setResult(RESULT_OK, data)
                    }

                } catch (e: Exception) {
                    Log.e(TAG, e.message.toString())
                } finally {
                    finish()
                }
            } ?: finish()
        }
    }

    private fun getRealPathFromURI(contentUri: Uri): String {
        val proj = arrayOf(MediaStore.Images.Media.DATA)
        val loader = CursorLoader(this, contentUri, proj, null, null, null)
        val cursor: Cursor = loader.loadInBackground()
        val columnIndex: Int = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATA)
        cursor.moveToFirst()
        val result: String = cursor.getString(columnIndex)
        cursor.close()
        return result
    }

    private fun connect(identity: Identity?) {
        //We don't have to stop a discovery but it's nice to do if we have found the camera that we are looking for
        //cameraHandler.stopDiscovery(discoveryStatusListener)

        if (connectedIdentity != null) {
            Log.d(TAG, "connect(), in *this* code sample we only support one camera connection at the time")
            showMessage.show("connect(), in *this* code sample we only support one camera connection at the time")
            return
        }

        if (identity == null) {
            Log.d(TAG, "connect(), can't connect, no camera available")
            showMessage.show("no camera available")
            return
        }

        connectedIdentity = identity

        updateConnectionText(identity, "CONNECTING")
        //IF your using "USB_DEVICE_ATTACHED" and "usb-device vendor-id" in the Android Manifest
        // you don't need to request permission, see documentation for more information
        if (UsbPermissionHandler.isFlirOne(identity)) {
            usbPermissionHandler.requestFlirOnePermisson(identity, this, permissionListener)
        } else {
            doConnect(identity)
        }
    }

    private fun doConnect(identity: Identity) {
        Thread {
            try {
                cameraHandler.connect(identity, connectionStatusListener)
                runOnUiThread {
                    updateConnectionText(identity, "CONNECTED")
                    cameraHandler.startStream(streamDataListener)
                    binding.imvCapture.isEnabled = true
                }
            } catch (e: IOException) {
                runOnUiThread {
                    updateConnectionText(identity, "DISCONNECTED")
                }
            }
        }.start()
    }

    private val streamDataListener = object : CameraHandler.StreamDataListener {
        override fun images(dataHolder: FrameDataHolder?) {
            runOnUiThread {
                mBitmap = dataHolder?.msxBitmap
                binding.imvCamera.setImageBitmap(dataHolder?.msxBitmap)
            }
        }

        override fun images(msxBitmap: Bitmap?, dcBitmap: Bitmap?) {
            try {
                framesBuffer.put(FrameDataHolder(msxBitmap, dcBitmap))
            } catch (e: InterruptedException) {
                Log.e(TAG, "images(), unable to add incoming to frames buffer, exception: $e")
            }

            runOnUiThread {
                Log.d(TAG, "framebuffer size: ${framesBuffer.size}")
                val poll = framesBuffer.poll()
                mBitmap = poll?.msxBitmap
                binding.imvCamera.setImageBitmap(poll?.msxBitmap)
            }
        }
    }

    private val permissionListener = object : UsbPermissionHandler.UsbPermissionListener {
        override fun permissionGranted(identity: Identity) {
            doConnect(identity)
        }

        override fun permissionDenied(identity: Identity) {
            showMessage.show("Permission was denied for identity")
        }

        override fun error(errorType: UsbPermissionHandler.UsbPermissionListener.ErrorType?, identity: Identity?) {
            showMessage.show("Error when asking for permission for FLIR ONE, error:$errorType identity:$identity")
        }
    }

    private val discoveryStatusListener = object : CameraHandler.DiscoveryStatus {
        override fun started() {
            showMessage.show("Status discovering")
        }

        override fun stopped() {
            showMessage.show("Status not discovering")
        }
    }

    private val connectionStatusListener = ConnectionStatusListener {
        Log.d(TAG, "onDisconnected errorCode:$it")
        runOnUiThread { updateConnectionText(connectedIdentity, "DISCONNECTED") }
    }

    private val cameraDiscoveryListener = object : DiscoveryEventListener {
        override fun onCameraFound(identity: Identity?) {
            runOnUiThread {
                cameraHandler.add(identity)
                connect(cameraHandler.flirOne)
            }
        }

        override fun onDiscoveryError(communicationInterface: CommunicationInterface?, errorCode: ErrorCode?) {
            runOnUiThread {
                showMessage.show("onDiscoveryError communicationInterface:$communicationInterface errorCode:$errorCode")
            }
        }
    }

    private fun updateConnectionText(identity: Identity?, status: String) {
        binding.txvMessage.text = "${identity?.deviceId} $status"
    }

    private val showMessage: ShowMessage = object : ShowMessage {
        override fun show(message: String?) {
            binding.txvMessage.text = message
        }
    }
} 