package com.zoinla.flir_plugin

import android.app.Activity
import android.content.Intent
import android.content.pm.PackageManager
import android.media.MediaScannerConnection
import android.net.Uri
import android.provider.MediaStore
import android.util.Log
import androidx.annotation.VisibleForTesting
import androidx.core.app.ActivityCompat
import androidx.core.content.FileProvider
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.PluginRegistry
import java.io.File
import java.io.IOException
import java.util.*

class FlirDelegate :
        PluginRegistry.ActivityResultListener,
        PluginRegistry.RequestPermissionsResultListener {
    @VisibleForTesting val REQUEST_CODE_STORAGE_PERMISSION = 2222
    @VisibleForTesting val REQUEST_CODE_TAKE_IMAGE_WITH_CAMERA = 2234
    @VisibleForTesting var fileProviderName: String? = null

    private var activity: Activity? = null
    private var externalFilesDirectory: File? = null
    private var imageResizer: ImageResizer? = null
    private var cache: FlirCache? = null
    private var permissionManager: PermissionManager? = null
    private var intentResolver: IntentResolver? = null
    private var fileUriResolver: FileUriResolver? = null
    private var fileUtils: FileUtils? = null

    interface PermissionManager {
        fun isPermissionGranted(permissionName: String): Boolean
        fun askForPermission(permissionName: String, requestCode: Int)
    }

    interface IntentResolver {
        fun resolveActivity(intent: Intent): Boolean
    }

    interface FileUriResolver {
        fun resolveFileProviderUriForFile(fileProviderName: String, imageFile: File): Uri
        fun getFullImagePath(imageUri: Uri?, listener: OnPathReadyListener?)
    }

    interface OnPathReadyListener {
        fun onPathReady(path: String?)
    }

    private var pendingCameraMediaUri: Uri? = null
    private var pendingResult: MethodChannel.Result? = null
    private var methodCall: MethodCall? = null

    constructor(activity: Activity,
                externalFilesDirectory: File,
                imageResizer: ImageResizer,
                cache: FlirCache) :
        this(
            activity,
            externalFilesDirectory,
            imageResizer,
            null,
            null,
            cache,
            object : PermissionManager {
                override fun isPermissionGranted(permissionName: String): Boolean {
                    return (ActivityCompat.checkSelfPermission(activity, permissionName)
                            == PackageManager.PERMISSION_GRANTED)
                }

                override fun askForPermission(permissionName: String, requestCode: Int) {
                    ActivityCompat.requestPermissions(activity, arrayOf(permissionName), requestCode)
                }
            },
            object : IntentResolver {
                override fun resolveActivity(intent: Intent): Boolean {
                    return intent.resolveActivity(activity.packageManager) != null
                }
            },
            object : FileUriResolver {
                override fun resolveFileProviderUriForFile(fileProviderName: String, imageFile: File): Uri {
                    return FileProvider.getUriForFile(activity, fileProviderName, imageFile)
                }

                override fun getFullImagePath(imageUri: Uri?, listener: OnPathReadyListener?) {
                    MediaScannerConnection.scanFile(
                            activity, arrayOf(if (imageUri != null) imageUri.path else ""),
                            null
                    ) { path, uri -> listener?.onPathReady(path) }
                }
            },
            FileUtils()
        )

    /**
     * This constructor is used exclusively for testing; it can be used to provide mocks to final
     * fields of this class. Otherwise those fields would have to be mutable and visible.
     */
    @VisibleForTesting
    constructor(
            activity: Activity,
            externalFilesDirectory: File,
            imageResizer: ImageResizer,
            result: MethodChannel.Result?,
            methodCall: MethodCall?,
            cache: FlirCache,
            permissionManager: PermissionManager?,
            intentResolver: IntentResolver?,
            fileUriResolver: FileUriResolver?,
            fileUtils: FileUtils?) {
        this.activity = activity
        this.externalFilesDirectory = externalFilesDirectory
        this.imageResizer = imageResizer
        this.fileProviderName = activity.packageName + ".flir_provider"
        this.pendingResult = result
        this.methodCall = methodCall
        this.permissionManager = permissionManager!!
        this.intentResolver = intentResolver!!
        this.fileUriResolver = fileUriResolver!!
        this.fileUtils = fileUtils!!
        this.cache = cache
    }

    fun saveStateBeforeResult() {
        methodCall?.let {
            cache?.saveDimensionWithMethodCall(it)
            pendingCameraMediaUri?.let { uri ->
                cache?.savePendingCameraMediaUriPath(uri)
            }
        }
    }

    fun retrieveLostImage(result: MethodChannel.Result) {
        val resultMap = cache?.getCacheMap()?.toMutableMap()
        resultMap?.let {
            val path = resultMap[FlirCache.MAP_KEY_PATH] as String?
            path?.let {
                val maxWidth = resultMap[FlirCache.MAP_KEY_MAX_WIDTH] as Double
                val maxHeight = resultMap[FlirCache.MAP_KEY_MAX_HEIGHT] as Double
                val imageQuality = resultMap[FlirCache.MAP_KEY_IMAGE_QUALITY] as Int? ?: 100

                val newPath = imageResizer?.resizeImageIfNeeded(path, maxWidth, maxHeight, imageQuality)
                resultMap[FlirCache.MAP_KEY_PATH] = newPath
            }

            if (resultMap.isEmpty())
                result.success(null)
            else
                result.success(resultMap)
        }

        cache?.clear()
    }

    fun takeImageWithCamera(methodCall: MethodCall, result: MethodChannel.Result) {
        if(!setPendingMethodCallAndResult(methodCall, result)) {
            finishWithAlreadyActiveError(result)
            return
        }

        launchFlirCameraIntent()
    }

    private fun launchFlirCameraIntent() {
        //val intent = Intent(MediaStore.ACTION_IMAGE_CAPTURE)
        val intent = Intent(activity, FlirCameraActivity::class.java)
        val imageFile: File = createTemporaryWritableImageFile()
        pendingCameraMediaUri = Uri.parse("file:" + imageFile.absolutePath)

        val imageUri = fileUriResolver?.resolveFileProviderUriForFile(fileProviderName!!, imageFile)
        intent.putExtra(MediaStore.EXTRA_OUTPUT, imageUri)
        grantUriPermissions(intent, imageUri!!)

        activity?.startActivityForResult(intent, REQUEST_CODE_TAKE_IMAGE_WITH_CAMERA)
    }

    private fun createTemporaryWritableImageFile(): File {
        return createTemporaryWritableFile(".jpg")
    }

    private fun createTemporaryWritableFile(suffix: String): File {
        val filename = UUID.randomUUID().toString()
        return try {
                File.createTempFile(filename, suffix, externalFilesDirectory)
            } catch (e: IOException) {
                throw RuntimeException(e)
            }
    }

    private fun grantUriPermissions(intent: Intent, imageUri: Uri) {
        val packageManager = activity?.packageManager
        val compatibleActivities = packageManager?.queryIntentActivities(intent, PackageManager.MATCH_DEFAULT_ONLY)
        compatibleActivities?.let {
            for (info in compatibleActivities) {
                activity?.grantUriPermission(
                        info.activityInfo.packageName,
                        imageUri,
                        Intent.FLAG_GRANT_READ_URI_PERMISSION or Intent.FLAG_GRANT_WRITE_URI_PERMISSION)
            }
        }
    }

    private fun handleCameraImageResult(resultCode: Int) {
        if (resultCode == Activity.RESULT_OK) {
            fileUriResolver?.getFullImagePath(
                    pendingCameraMediaUri ?: Uri.parse(cache?.retrievePendingCameraMediaUriPath()),
                    object : OnPathReadyListener {
                        override fun onPathReady(path: String?) {
                            path?.let { handleImageResult(path, true) }
                        }
                    })
            return
        }

        finishWithSuccess(null)
    }

    private fun handleImageResult(path: String, shouldDeleteOriginalIfScaled: Boolean) {
        methodCall?.let {
            val maxWidth = it.argument<Double>("maxWidth")
            val maxHeight = it.argument<Double>("maxHeight")
            val imageQuality = it.argument<Int>("imageQuality")
            val finalImagePath = imageResizer?.resizeImageIfNeeded(path, maxWidth, maxHeight, imageQuality)
            finishWithSuccess(finalImagePath)

            //delete original file if scaled
            if (finalImagePath != null && finalImagePath != path && shouldDeleteOriginalIfScaled) {
                File(path).delete()
            }
        } ?: finishWithSuccess(path)
    }

    private fun setPendingMethodCallAndResult(
            methodCall: MethodCall, result: MethodChannel.Result): Boolean {
        if (pendingResult != null) {
            return false
        }
        this.methodCall = methodCall
        pendingResult = result

        // Clean up cache if a new image picker is launched.
        cache?.clear()
        return true
    }

    private fun finishWithSuccess(imagePath: String?) {
        if (pendingResult == null) {
            cache?.saveResult(imagePath, null, null)
            return
        }
        pendingResult?.success(imagePath)
        clearMethodCallAndResult()
    }

    private fun finishWithAlreadyActiveError(result: MethodChannel.Result) {
        result.error("already_active", "Image picker is already active", null)
    }

    private fun finishWithError(errorCode: String, errorMessage: String) {
        if (pendingResult == null) {
            cache?.saveResult(null, errorCode, errorMessage)
            return
        }

        pendingResult?.error(errorCode, errorMessage, null)
        clearMethodCallAndResult()
    }

    private fun clearMethodCallAndResult() {
        methodCall = null
        pendingResult = null
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        when (requestCode) {
            REQUEST_CODE_TAKE_IMAGE_WITH_CAMERA ->
                handleCameraImageResult(resultCode)
            else ->
                return false
        }

        return true
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>?, grantResults: IntArray?): Boolean {
        TODO("Not yet implemented")
    }
}