package com.zoinla.flir_plugin

import android.app.Activity
import android.app.Application
import android.os.Bundle
import android.os.Environment
import android.os.Handler
import android.os.Looper
import androidx.annotation.NonNull
import androidx.annotation.VisibleForTesting
import androidx.lifecycle.DefaultLifecycleObserver
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleOwner
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry

/** FlirPlugin */
class FlirPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  private class LifeCycleObserver(private var thisActivity: Activity,
                                  private var delegate: FlirDelegate):
          Application.ActivityLifecycleCallbacks, DefaultLifecycleObserver {
    override fun onActivityCreated(activity: Activity, savedInstanceState: Bundle?) { }

    override fun onActivityStarted(activity: Activity) { }

    override fun onActivityResumed(activity: Activity) { }

    override fun onActivityPaused(activity: Activity) { }

    override fun onActivityStopped(activity: Activity) {
      if (thisActivity == activity)
        delegate.saveStateBeforeResult()
    }

    override fun onActivitySaveInstanceState(activity: Activity, outState: Bundle) { }

    override fun onActivityDestroyed(activity: Activity) {
      if (thisActivity == activity && activity.applicationContext != null)
        (activity.applicationContext as Application)
                .unregisterActivityLifecycleCallbacks(this)
    }

    override fun onStop(owner: LifecycleOwner) { onActivityStopped(thisActivity) }

    override fun onDestroy(owner: LifecycleOwner) { onActivityDestroyed(thisActivity) }
  }

  private var channel : MethodChannel? = null
  private var delegate: FlirDelegate? = null
  private var pluginBinding: FlutterPlugin.FlutterPluginBinding? = null
  private var activityBinding: ActivityPluginBinding? = null
  private var application: Application? = null
  private var activity: Activity? = null
  private var lifecycle: Lifecycle? = null
  private var observer: LifeCycleObserver? = null

  /**
   * Default Constructor for plugin.
   */
  constructor() { }

  @VisibleForTesting
  constructor(delegate: FlirDelegate, activity: Activity) {
    this.delegate = delegate
    this.activity = activity
  }

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
//    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "flir_plugin")
//    channel.setMethodCallHandler(this)

    pluginBinding = flutterPluginBinding
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull rawResult: Result) {
//    if (call.method == "getPlatformVersion") {
//      result.success("Android ${android.os.Build.VERSION.RELEASE}")
//    } else {
//      result.notImplemented()
//    }
    if (activity == null) {
      rawResult.error("no_activity", "image_picker plugin requires a foreground activity.", null)
      return
    }

    val result = MethodResultWrapper(rawResult)
    when(call.method) {
      METHOD_CALL_FLIR_CAMERA ->
        delegate?.takeImageWithCamera(call, result)
      METHOD_CALL_RETRIEVE ->
        delegate?.retrieveLostImage(result)
      else ->
        throw IllegalArgumentException("Unknown method " + call.method)
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
//    channel.setMethodCallHandler(null)

    pluginBinding = null
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activityBinding = binding
    setup(
            pluginBinding?.binaryMessenger,
            pluginBinding?.applicationContext as Application,
            activityBinding?.activity!!,
            null,
            activityBinding)
  }

  override fun onDetachedFromActivityForConfigChanges() {
    onDetachedFromActivity()
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    onAttachedToActivity(binding)
  }

  override fun onDetachedFromActivity() {
    tearDown()
  }

  @Suppress("DEPRECATION")
  private fun setup(messenger: BinaryMessenger?,
                    application: Application?,
                    activity: Activity,
                    registrar: PluginRegistry.Registrar?,
                    activityBinding: ActivityPluginBinding?){
    this.activity = activity
    this.application = application
    this.delegate = constructFlirDelegate(activity)
    channel = MethodChannel(messenger, "flir_plugin")
    channel?.setMethodCallHandler(this)
    observer = LifeCycleObserver(activity, delegate!!)
    if (registrar != null) {
      // V1 embedding setup for activity listeners
      application?.registerActivityLifecycleCallbacks(observer)
      registrar.addActivityResultListener(delegate)
      registrar.addRequestPermissionsResultListener(delegate)
    } else {
      // V2 embedding setup for activity listeners.
      activityBinding?.addActivityResultListener(delegate!!)
      activityBinding?.addRequestPermissionsResultListener(delegate!!)
      activityBinding?.let {
        lifecycle = FlutterLifecycleAdapter.getActivityLifecycle(activityBinding)
        lifecycle?.addObserver(observer!!)
      }
    }
  }

  private fun tearDown() {
    delegate?.let {
      activityBinding?.removeActivityResultListener(it)
      activityBinding?.removeRequestPermissionsResultListener(it)
    }
    activityBinding = null
    observer?.let {
      lifecycle?.removeObserver(it)
    }
    lifecycle = null
    delegate = null
    channel?.setMethodCallHandler(null)
    channel = null
    application?.unregisterActivityLifecycleCallbacks(observer)
    application = null
  }

  private fun constructFlirDelegate(setupActivity: Activity): FlirDelegate {
    val cache = FlirCache(setupActivity)
    val externalFilesDirectory = setupActivity.getExternalFilesDir(Environment.DIRECTORY_PICTURES)
    val exifDataCopier = ExifDataCopier()
    val imageResizer = ImageResizer(externalFilesDirectory!!, exifDataCopier)
    return FlirDelegate(setupActivity, externalFilesDirectory, imageResizer, cache)
  }

  companion object {
    const val METHOD_CALL_FLIR_CAMERA = "flirCamera"
    private const val METHOD_CALL_RETRIEVE = "retrieve"

    @Suppress("DEPRECATION")
    fun registerWith(registrar: PluginRegistry.Registrar) {
      registrar.activity()?.let {
        val activity = it
        var application: Application? = null
        registrar.context()?.let {
          application = registrar.context().applicationContext as Application
        }
        val plugin = FlirPlugin()
        plugin.setup(registrar.messenger(), application, activity, registrar, null)
      }
    }

    private class MethodResultWrapper(
            private var methodResult: Result): Result {
      private var handler = Handler(Looper.getMainLooper())

      override fun success(result: Any?) {
        handler.post { methodResult.success(result) }
      }

      override fun error(errorCode: String?, errorMessage: String?, errorDetails: Any?) {
        handler.post { methodResult.error(errorCode, errorMessage, errorDetails) }
      }

      override fun notImplemented() {
        handler.post { methodResult.notImplemented() }
      }
    }
  }
}
