package com.zoinla.flir_plugin

import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.util.Log
import java.io.ByteArrayOutputStream
import java.io.File
import java.io.FileOutputStream
import java.io.IOException

class ImageResizer(
        private var externalFilesDirectory: File,
        private var exifDataCopier: ExifDataCopier) {

    /**
     * If necessary, resizes the image located in imagePath and then returns the path for the scaled
     * image.
     *
     *
     * If no resizing is needed, returns the path for the original image.
     */
    fun resizeImageIfNeeded(
            imagePath: String,
            maxWidth: Double?,
            maxHeight: Double?,
            imageQuality: Int?): String? {
        val bmp = decodeFile(imagePath) ?: return null
        val shouldScale = maxWidth != null || maxHeight != null || isImageQualityValid(imageQuality)
        return if (!shouldScale) {
            imagePath
        } else try {
            val pathParts = imagePath.split("/").toTypedArray()
            val imageName = pathParts[pathParts.size - 1]
            val file = resizedImage(bmp, maxWidth, maxHeight, imageQuality, imageName)
            copyExif(imagePath, file.path)
            file.path
        } catch (e: IOException) {
            throw RuntimeException(e)
        }
    }

    @Throws(IOException::class)
    private fun resizedImage(
            bmp: Bitmap, maxWidth: Double?, maxHeight: Double?, imageQuality: Int?, outputImageName: String): File {
        var imageQuality = imageQuality!!
        val originalWidth = bmp.width * 1.0
        val originalHeight = bmp.height * 1.0
        if (!isImageQualityValid(imageQuality)) {
            imageQuality = 100
        }
        val hasMaxWidth = maxWidth != null
        val hasMaxHeight = maxHeight != null
        var width = if (hasMaxWidth) Math.min(originalWidth, maxWidth!!) else originalWidth
        var height = if (hasMaxHeight) Math.min(originalHeight, maxHeight!!) else originalHeight
        val shouldDownscaleWidth = hasMaxWidth && maxWidth!! < originalWidth
        val shouldDownscaleHeight = hasMaxHeight && maxHeight!! < originalHeight
        val shouldDownscale = shouldDownscaleWidth || shouldDownscaleHeight
        if (shouldDownscale) {
            val downscaledWidth = height / originalHeight * originalWidth
            val downscaledHeight = width / originalWidth * originalHeight
            if (width < height) {
                if (!hasMaxWidth) {
                    width = downscaledWidth
                } else {
                    height = downscaledHeight
                }
            } else if (height < width) {
                if (!hasMaxHeight) {
                    height = downscaledHeight
                } else {
                    width = downscaledWidth
                }
            } else {
                if (originalWidth < originalHeight) {
                    width = downscaledWidth
                } else if (originalHeight < originalWidth) {
                    height = downscaledHeight
                }
            }
        }
        val scaledBmp = createScaledBitmap(bmp, width.toInt(), height.toInt(), false)
        return createImageOnExternalDirectory("/scaled_$outputImageName", scaledBmp, imageQuality)
    }

    private fun createFile(externalFilesDirectory: File?, child: String): File {
        val image = File(externalFilesDirectory, child)
        if (!image.parentFile.exists()) {
            image.parentFile.mkdirs()
        }
        return image
    }

    @Throws(IOException::class)
    private fun createOutputStream(imageFile: File): FileOutputStream {
        return FileOutputStream(imageFile)
    }

    private fun copyExif(filePathOri: String, filePathDest: String) {
        exifDataCopier.copyExif(filePathOri, filePathDest)
    }

    private fun decodeFile(path: String): Bitmap? {
        return BitmapFactory.decodeFile(path)
    }

    private fun createScaledBitmap(bmp: Bitmap, width: Int, height: Int, filter: Boolean): Bitmap {
        return Bitmap.createScaledBitmap(bmp, width, height, filter)
    }

    private fun isImageQualityValid(imageQuality: Int?): Boolean {
        return imageQuality != null && imageQuality > 0 && imageQuality < 100
    }

    @Throws(IOException::class)
    private fun createImageOnExternalDirectory(name: String, bitmap: Bitmap, imageQuality: Int): File {
        val outputStream = ByteArrayOutputStream()
        val saveAsPNG = bitmap.hasAlpha()
        if (saveAsPNG) {
            Log.d(
                    "ImageResizer",
                    "image_picker: compressing is not supported for type PNG. Returning the image with original quality")
        }
        bitmap.compress(
                if (saveAsPNG) Bitmap.CompressFormat.PNG else Bitmap.CompressFormat.JPEG,
                imageQuality,
                outputStream)
        val imageFile = createFile(externalFilesDirectory, name)
        val fileOutput = createOutputStream(imageFile)
        fileOutput.write(outputStream.toByteArray())
        fileOutput.close()
        return imageFile
    }
}